<?php

return [

    'title' => 'သင့်အီးမေးလ်လိပ်စာကို အတည်ပြုပါ',

    'heading' => 'သင့်အီးမေးလ်လိပ်စာကို အတည်ပြုပါ',

    'actions' => [

        'resend_notification' => [
            'label' => 'ပြန်လည်ပို့ပေးပါ',
        ],

    ],

    'messages' => [
        'notification_not_received' => 'ကျွန်ုပ်တို့ပို့လိုက်သော အီးမေးလ်ကို မရရှိပါသလား?',
        'notification_sent' => 'သင့်အီးမေးလ်လိပ်စာ အတည်ပြုရန်အတွက် လမ်းညွှန်ချက်များပါဝင်သော အီးမေးလ်ကို :email သို့ ပို့ပြီးဖြစ်ပါသည်။',
    ],

    'notifications' => [

        'notification_resent' => [
            'title' => 'အီးမေးလ်အသစ် ပြန်လည်ပို့ပြီးပါပြီ။',
        ],

        'notification_resend_throttled' => [
            'title' => 'ပြန်လည်ပို့ရန် ကြိုးပမ်းမှု များလွန်းနေပါသည်',
            'body' => 'ကျေးဇူးပြု၍ :seconds စက္ကန့် မှာ ထပ်မံကြိုးစားပါ။',
        ],

    ],

];
