<?php

return [

    'entries' => [

        'text' => [

            'actions' => [
                'collapse_list' => 'Zobraziť o :count menej',
                'expand_list' => 'Zobraziť o :count viac',
            ],

            'more_list_items' => 'a o :count viac',

        ],

        'key_value' => [

            'columns' => [

                'key' => [
                    'label' => 'Kľúč',
                ],

                'value' => [
                    'label' => 'Hodnota',
                ],

            ],

            'placeholder' => 'Žiadne záznamy',

        ],

    ],

];
