<?php

return [

    'title' => ':label 만들기',

    'breadcrumb' => '만들기',

    'form' => [

        'actions' => [

            'cancel' => [
                'label' => '취소',
            ],

            'create' => [
                'label' => '만들기',
            ],

            'create_another' => [
                'label' => '계속 만들기',
            ],

        ],

    ],

    'notifications' => [

        'created' => [
            'title' => '생성 완료',
        ],

    ],

];
