<?php

return [

    'direction' => 'ltr',

    'actions' => [

        'billing' => [
            'label' => 'Obuna boshqaruvi',
        ],

        'logout' => [
            'label' => 'Chiqish',
        ],

        'open_database_notifications' => [
            'label' => 'Bildirishnomalarni ochish',
        ],

        'open_user_menu' => [
            'label' => 'Foydalanuvchi menyusi',
        ],

        'sidebar' => [

            'collapse' => [
                'label' => 'Yon panelni yig\'ish',
            ],

            'expand' => [
                'label' => 'Yon panelni yoyish',
            ],

        ],

        'theme_switcher' => [

            'dark' => [
                'label' => 'To\'q rejimni ishlatish',
            ],

            'light' => [
                'label' => 'Yorqin rejimni ishlatish',
            ],

            'system' => [
                'label' => 'Tizim rejimidan foydalanish',
            ],

        ],

    ],

];
