<?php

return [

    'single' => [

        'label' => '関連付け解除',

        'modal' => [

            'heading' => ':labelの関連付け解除',

            'actions' => [

                'dissociate' => [
                    'label' => '関連付け解除',
                ],

            ],

        ],

        'notifications' => [

            'dissociated' => [
                'title' => '関連付け解除しました',
            ],

        ],

    ],

    'multiple' => [

        'label' => '選択中を関連付け解除',

        'modal' => [

            'heading' => '選択中の:labelを関連付け解除',

            'actions' => [

                'dissociate' => [
                    'label' => '関連付け解除',
                ],

            ],

        ],

        'notifications' => [

            'dissociated' => [
                'title' => '関連付け解除しました',
            ],

        ],

    ],

];
