<?php

return [

    'confirmation' => '本当に実行しますか？',

    'actions' => [

        'cancel' => [
            'label' => 'キャンセル',
        ],

        'confirm' => [
            'label' => '確定',
        ],

        'submit' => [
            'label' => '完了',
        ],

    ],

];
