<?php

return [

    'label' => 'Profilis',

    'form' => [

        'email' => [
            'label' => 'El. paštas',
        ],

        'name' => [
            'label' => 'Vardas',
        ],

        'password' => [
            'label' => 'Naujas slaptažodis',
        ],

        'password_confirmation' => [
            'label' => 'Patvirtinkite naują slaptažodį',
        ],

        'actions' => [

            'save' => [
                'label' => 'Išsaugoti',
            ],

        ],

    ],

    'notifications' => [

        'saved' => [
            'title' => 'Išsaugota',
        ],

    ],

    'actions' => [

        'cancel' => [
            'label' => 'atgal',
        ],

    ],

];
