<?php

return [

    'modal' => [

        'heading' => 'Notifiche',

        'actions' => [

            'clear' => [
                'label' => 'Pulisci',
            ],

            'mark_all_as_read' => [
                'label' => 'Imposta tutto come letto',
            ],

        ],

        'empty' => [
            'heading' => 'Nessuna notifica',
            'description' => 'Si prega di controllare più tardi',
        ],

    ],

];
