<?php

return [

    'confirmation' => 'Ali ste prepričani, da želite to narediti?',

    'actions' => [

        'cancel' => [
            'label' => 'Prekliči',
        ],

        'confirm' => [
            'label' => 'Potrdi',
        ],

        'submit' => [
            'label' => 'Pošlji',
        ],

    ],

];
