<?php

return [

    'direction' => 'ltr',

    'actions' => [

        'billing' => [
            'label' => 'Upravljaj pretplatom',
        ],

        'logout' => [
            'label' => 'Odjava',
        ],

        'open_database_notifications' => [
            'label' => 'Otvori obavijesti',
        ],

        'open_user_menu' => [
            'label' => 'Korisnički izbornik',
        ],

        'sidebar' => [

            'collapse' => [
                'label' => 'Skupi bočnu traku',
            ],

            'expand' => [
                'label' => 'Proširi bočnu traku',
            ],

        ],

        'theme_switcher' => [

            'dark' => [
                'label' => 'Omogući tamnu temu',
            ],

            'light' => [
                'label' => 'Omogući svijetlu temu',
            ],

            'system' => [
                'label' => 'Omogući temu sustava',
            ],

        ],

    ],

    'avatar' => [
        'alt' => 'Avatar od :name',
    ],

    'logo' => [
        'alt' => ':name logo',
    ],

];
